"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initializeChangelog = exports.addCommitizen = exports.addRemoteOrigin = exports.initializeProject = exports.createNewRepo = exports.fullPush = exports.push = exports.commit = exports.addAll = exports.checkout = void 0;

var _exec = require("../exec");

var _fetch = require("../fetch");

var _print = require("../print");

var _utils = require("./utils");

const REPO_ALREADY_EXISTS = 422;

const checkout = async (branch, isNew = false) => {
  await (0, _exec.execute)(`git checkout ${isNew ? '-b' : ''} ${branch}`, `Checking out ${branch}`);
};

exports.checkout = checkout;

const addAll = async branch => {
  await (0, _exec.execute)(`git add -A`, `Adding all updated files to ${branch}`);
}; // eslint-disable-next-line max-params


exports.addAll = addAll;

const commit = async (type, scope, message, closes, breaking) => {
  const commitMessage = (0, _utils.generateStandardCommitMessage)(type, scope, message, closes, breaking);
  await (0, _exec.execute)(`git commit -m "${commitMessage.title}" -m "${commitMessage.description}"`, `Adding commit message:
    ${commitMessage.title}, ${commitMessage.description}`);
};

exports.commit = commit;

const push = async (branch, link = false) => {
  await (0, _exec.execute)(`git push ${link ? '-u' : ''} origin ${branch}`, `Pushing updated files to ${branch}`);
};

exports.push = push;

const fullPush = // eslint-disable-next-line
async (branch, newBranch, type, scope, message, closes, breaking = false) => {
  await checkout(branch, newBranch);
  await addAll(branch);
  await commit(type, scope, message, closes, breaking);
  await push(branch, newBranch);
};

exports.fullPush = fullPush;

const createNewRepo = async (projectName, token) => {
  try {
    await (0, _fetch.fetch)('https://api.github.com/user/repos', 'POST', JSON.stringify({
      name: projectName
    }), token, null, true);
  } catch ({
    message,
    code
  }) {
    if (code === REPO_ALREADY_EXISTS) {
      (0, _print.printWarning)('Repository already exists');
      throw new Error('Repository already exists');
    } else {
      (0, _print.printError)('Unable to create new repository');
      throw new Error(`${code} ${message}`);
    }
  }
};

exports.createNewRepo = createNewRepo;

const initializeProject = async () => {
  await (0, _exec.execute)('git init .', 'Initializing the git project');
};

exports.initializeProject = initializeProject;

const addRemoteOrigin = async (repoOwner, projectName) => {
  await (0, _exec.execute)(`git remote add origin git@github.com:${repoOwner}/${projectName}.git`, `Adding remote origin git@github.com:${repoOwner}/${projectName}.git`);
  (0, _print.printWarning)(`Note: the access token provided must come from the account ${repoOwner}`);
};

exports.addRemoteOrigin = addRemoteOrigin;

const addCommitizen = async () => {
  (0, _print.printWarning)('...please wait. This may take a few minutes...');
  await (0, _exec.execute)(`npm i -D commitizen`, 'Installing commitizen as a dev dependency');
};

exports.addCommitizen = addCommitizen;

const initializeChangelog = async () => {
  (0, _print.printWarning)('...please wait. This may take a few minutes...');
  await (0, _exec.execute)( // eslint-disable-next-line
  `npx commitizen --force init cz-conventional-changelog --save-dev --save-exact`, 'Setting cz-conventional-changelog as changelog');
};

exports.initializeChangelog = initializeChangelog;